<?php

namespace WAAVE\Complete\Controller\Direct;

use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Checkout\Model\Session;
use Magento\Sales\Model\OrderFactory;
use Magento\Sales\Model\Order;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;
use WAAVE\Complete\Gateway\Config\Direct\Config;
use WAAVE\Complete\Helper\Data as DataHelper;
use WAAVE\Compliance\Model\ConfigurationFactory;

class Index extends Action
{
    /**
     * @var Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var Magento\Checkout\Model\Session
     */
    private $checkoutSession;

    /**
     * @var Magento\Sales\Model\OrderFactory
     */
    private $orderFactory;

    /**
     * @var WAAVE\Compliance\Model\ConfigurationFactory
     */
    private $configurationFactory;

    /**
     * @var WAAVE\Complete\Gateway\Config\Direct\Config
     */
    private $gatewayConfig;

    /**
     * @var DataHelper
     */
    private $helper;

    /**
     * @var Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var RemoteAddress
     */
    private $remoteAddress;

    public function __construct(
        Config $gatewayConfig,
        Session $checkoutSession,
        StoreManagerInterface $storeManager,
        OrderFactory $orderFactory,
        ConfigurationFactory $configurationFactory,
        LoggerInterface $logger,
        DataHelper $helper,
        RemoteAddress $remoteAddress,
        Context $context
    ) {
        $this->checkoutSession      = $checkoutSession;
        $this->orderFactory         = $orderFactory;
        $this->configurationFactory = $configurationFactory;
        $this->gatewayConfig        = $gatewayConfig;
        $this->storeManager         = $storeManager;
        $this->helper               = $helper;
        $this->remoteAddress        = $remoteAddress;

        parent::__construct($context);
    }

    /**
     * Get checkout session
     */
    protected function getCheckoutSession()
    {
        return $this->checkoutSession;
    }

    /**
     * Get configuration factory
     */
    protected function getConfiguration()
    {
        return $this->configurationFactory->create()->getCollection()->getFirstItem();
    }

    /**
     * Get logger
     */
    protected function getLogger()
    {
        return $this->logger;
    }

    /**
     * Get order
     */
    protected function getOrder()
    {
        $orderId = $this->checkoutSession->getLastRealOrderId();

        if (!isset($orderId)) {
            return null;
        }

        $order = $this->orderFactory->create()->loadByIncrementId($orderId);

        if (!$order->getId()) {
            return null;
        }

        return $order;
    }

    /**
     * Execute
     */
    public function execute()
    {
        $order = $this->getOrder();

        if ($order->getState() != Order::STATE_NEW) {
            $this->_redirect('checkout/onepage/failure', ['_secure'=> false]);
            return;
        }

        $baseUrl = $this->storeManager->getStore()->getBaseUrl();

        $returnUrl   = $baseUrl . 'checkout/onepage/success';
        $cancelUrl   = $baseUrl . 'waave_complete/checkout/cancel';
        $callbackUrl = $baseUrl . 'rest/V1/waave_complete/callback';

        $venueId    = $this->getConfiguration()->getData('venue_id');
        $privateKey = $this->getConfiguration()->getPrivateKey();

        $orderBilling  = $order->getBillingAddress();
        $countryId = $orderBilling->getCountryId();
        $billing  = [
            'email'      => $order->getCustomerEmail(),
            'first_name' => $order->getCustomerFirstname(),
            'last_name'  => $order->getCustomerLastname(),
            'phone'      => $orderBilling->getTelephone(),
            'phone_code' => $this->helper->getPhoneCode($countryId),
            'address_1'  => $orderBilling->getStreetLine(1),
            'city'       => $orderBilling->getCity(),
            'postcode'   => $orderBilling->getPostcode(),
            'state'      => $orderBilling->getRegion(),
            'country'    => $countryId
        ];

        $orderShipping = $order->getShippingAddress();
        $countryId = $orderShipping->getCountryId();
        $shipping  = [
            'email'      => $order->getCustomerEmail(),
            'phone'      => $orderShipping->getTelephone(),
            'phone_code' => $this->helper->getPhoneCode($countryId),
            'address_1'  => $orderShipping->getStreetLine(1),
            'city'       => $orderShipping->getCity(),
            'postcode'   => $orderShipping->getPostcode(),
            'state'      => $orderShipping->getRegion(),
            'country'    => $countryId
        ];

        $params = array(
            'access_key'      => $this->getConfiguration()->getAccessKey(),
            'venue_id'        => $venueId,
            'return_url'      => $returnUrl,
            'callback_url'    => $callbackUrl,
            'cancel_url'      => $cancelUrl,
            'validation_code' => $this->checkoutSession->getValidationCode(),

            // Order details
            'amount'       => (string)$order->getTotalDue(),
            'reference_id' => (string)$order->getRealOrderId(),
            'currency'     => $order->getOrderCurrencyCode(),
            'customer_ip'  => $this->remoteAddress->getRemoteAddress(),

            'card_information' => [
                'card_token' => $this->getRequest()->getParam('token'),
            ],
            'billing'  => $billing,
            'shipping' => $shipping
        );

        $gatewayUrl  = $this->gatewayConfig->getGatewayURL();

        $body       = json_encode($params);
        $signature  = hash("sha256", $privateKey . $gatewayUrl . $body);

        $client = new \GuzzleHttp\Client();

        try {
            $response = $client->request('POST', $gatewayUrl, [
                'form_params' => $params,
                'timeout' => 300,
                'headers' => [
                    'X-Api-Signature' => $signature
                ]
            ]);
        } catch (\GuzzleHttp\Exception\ClientException $e) {
            $this->checkoutSession->setErrorMessage($e->getMessage());
            $this->_redirect('checkout/onepage/failure', ['_secure'=> false]);
            return;
        }

        $this->_redirect('checkout/onepage/success', ['_secure'=> false]);
    }

}
